/* 	Main.java
 * 
 * 	Created on: Oct 2,2002
 * 	Created by: Dennis Groenendijk
 * 
 * Copyright (c) 2002, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/*
 * @#History 
 * [When]       [Who]               [What] 
 * 25-nov-2006  dennis@brains2b.nl  Updated for new version of ArgumentMap
 * 27-06-2007   dennis@brains2b.nl  Cleaned up and desynchronized Map usage
 */
package org.brains2b.console;

import java.util.HashMap;
import java.util.Map;

/** Main is a class for applications that are depending on commandline argument, 
 *  like console applications.  
 *  <p>Main together with ArgumentMap supply a convenient way to define arguments, 
 * their relation and optionality and report if the supplied arguments when starting the application are meet
 * the defined argument requirements.
 *  <OL>The way to use this class is to extend your own class that contains a main method. 
 * <LI>Before you do anything to start the application create an ArgumentMap and add arguments and options to it. 
 * <LI>Call the setArgumentMap(ActionMap) supplying your argument map to it.
 * <LI>Afiter this you can call makeArgumentList(String[] args) with the arguments passed from main.
 * <LI>Alle recognized arguments are now available through getArgument(String name), supplying your
 * defined name for the argument.
 * <p><i>Note:</i>If no ArgumentMap is supplied to setArgumentMap(ArgumentMap) an argument list
 * is created with each argument supplied to makeArgumentList(String[]) with as name the numeric location of the 
 * argument starting at 0 and the value of the argument from that location in the array.
 * <p> All exceptions thrown by the ArgumentMap are processed through handleException(ArgumentException)
 * if you want to do anything other then writing it to System.err overwrite this function.
 * <p> The way I usually apply Main is:
 * <code><pre>
 * public class AConsole {
 * 	public AConsole() {
 * 		super();
 * 		init();
 * 		//other code
 * 	}
 * 	
 * 	public static void main	(String[] args) {
 * 		AConsole a=new AConsole();
 * 		a.makeArguments(args);
 * 		a.start();
 * 	}
 * 	
 * 	private void init() {
 * 		ArgumentMap arg=new ArgumentMap();
 * 		//add arguments
 * 		setArgumentMap(arg);
 * 	}
 * 	
 * 	private void start() {
 * 		//do your thing
 * 	}
 * }
 * </pre></code>
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version 0.10.1 [25-11-2006]
 * @see org.brains2b.console.ArgumentMap
 */
public class Main {

	private ArgumentMap m_argumentMap;
	private Map m_argumentList;
	/**
	 * Constructor for Main.
	 */
	public Main() {
		super();
	}

	
	/**
	 * gets  the ArgumentMap.
	 * @return ArgumentMap
	 */
	protected ArgumentMap getArgumentMap() {
		return m_argumentMap;
	}

	/**
	 * Sets the ArgumentMap.
	 * @param argumentMap ArgumentMap, the created ArgumentMap
	 */
	protected void setArgumentMap(ArgumentMap argumentMap) {
		m_argumentMap = argumentMap;
	}

	/**
	 * gets  the value of an argument defined by name.
	 * 
	 * @return String, the value belonging to argument name or null if argument was not defined
	 * @throws NullPointerException, thrown if there was no argument list made (forgot to call makeArgumentList()?)
	 */
	public String getArgument(String name) {
		if (m_argumentList==null) {
			throw new NullPointerException("There are no arguments set");
		}
		return (String) m_argumentList.get(name);
	}
	
	/**
	 * set arguments to the list of arguments.
	 * <p>This method does not check for validity of the argument and can possibly
	 * overwrite arguments set with makeArgumentList. Checking your arguments should be
	 * done by your own code.
	 * @param name
	 * @param value
	 */
	public void setArgument(String name, String value) {
		if (m_argumentList==null) {
			m_argumentList=new HashMap();
		}
		m_argumentList.put(name,value);
	}
		
	/**
	 * Makes the argument list on the basis of a defined ArgumentMap and the supplied
	 * array of arguments
	 * <p> If an ArgumentMap exists the arguments are passed to ArgumentMap.getList(String[])
	 *  <p> If no ArgumentMap exists a List is made with the numeric
	 * location of the argument as name and the corresponding value of the Array
	 * as value.
	 * @param args String[] An array of arguments
	 * @see org.brains2b.console.ArgumentMap#getList
	 */
	public void makeArgumentList(String[] args) {
		if (args==null) return;
		m_argumentList=new HashMap();
		if (m_argumentMap==null) {
			for (int i=0;i<args.length;i++) {
				m_argumentList.put(String.valueOf(i),args[i]);
			}
		} else {
			try {
				m_argumentList=m_argumentMap.getList(args);
			} catch (ArgumentException ex) {
				handleException(ex);
			}
		}
	}
	
	/**
	 * Default function for handling ArgumentException(s).
	 * <p>It writes the argument that caused the exception and the exception message to the System.err outstream and
	 * exits the VM with code <b>1</b>.
	 * <p>Overwrite if you want it to do something else.  
	 * <BR>Be sure to break normal operation if not exiting while arguments and options
	 * you set to required are not actually there and you have not set default values.
	 * @param ex ArgumentException
	 */
	protected void handleException(ArgumentException ex) {
		System.err.println(ex.getArgument() + ": " + ex.getMessage());
		System.exit(1);
	}
	
	
}
