/* 	DateHelper.java
 * 	Created on: Dec 11, 2002
 *	Copyright (c) 2002-2008, Brains2B.org
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that
 * the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and 
 * the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided with the distribution. 
 * 3. The name of the author may not be used to endorse or promote products derived from this software 
 * without specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* @#History
 * [When]		[Who]				[What]
 * 30-06-2007   dennis@brains2b.nl  changed eight numbers to ddmmyyyy, or mmddyyyy depending on Locale
 *                                  Implemented new iso date methods  
 * 08-11-2008	dennis@brains2b.nl	Made use of SimpleDateFormat threadsafe
 */
package org.brains2b.util;

import java.text.DateFormatSymbols;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

/**
 * A function library giving some extra help dealing with dates.
 * 
 * @author <A HREF="MAILTO:dennis@brains2b.nl">dennis@brains2b.nl</A>
 * @version $Revision: 1.13 $ $Date: 2009/09/15 08:09:58 $
 */
public class DateHelper {

	static class ThreadSafeDateFormat extends ThreadLocal {
		
		protected Object initialValue() {
			return new SimpleDateFormat();
		}
	}
	
    private static final Calendar m_cal = Calendar.getInstance();

    private static final int THIS_YEAR = m_cal.get(Calendar.YEAR);
    
    private static final Locale m_locale = Locale.getDefault();

    private final static String DEFAULT_DATE = "dd-MMM-yyyy";

    private static final String ISO_DATE = "yyyy-MM-dd";

    private static final String ISO_COMPLETE = "yyyy-MM-dd'T'HH:mm:ss.S";

    private static ThreadSafeDateFormat m_sdf = new ThreadSafeDateFormat();
    
    

    /**
     * gets a date formatted as String with pattern "dd-MMM-yyyy"
     * 
     * @param dt
     *            java.util.Date
     * @return String the formatted date, or null if Date was not given.
     */
    public final static String fromDate(Date dt) {
        if (dt == null)
            return null;
        ((SimpleDateFormat) m_sdf.get()).applyPattern(DEFAULT_DATE);
        return ((SimpleDateFormat) m_sdf.get()).format(dt);
    }

    /**
     * returns a Date from a String representing a Date
     * <p>
     * <UL>
     * This toDate function is basically a rule based system recognizing a Date
     * in the following patterns:
     * <LI>a six digit String is interpreted as <code>ddMMyy</code>
     * <LI>a String with a non character separator is interpreted as
     * <code>day-month-year</code>
     * <LI><code>day</code> can be either 1 or 2 digits, with or without a
     * leading 0
     * <LI><code>month</code> can be either 1 or 2 digits, with or without a
     * leading 0, a String representing the long name or a short name of the
     * month. The name of the month must be in the same language at which the VM
     * it is operating is running in <i>Locale.getDefault() is used to determine
     * this.</li>
     * <LI><code>year</code> can be 1 or 2 digits, with or without a leading
     * 0. This is interpreted as a year without a century, so the current
     * century is added. It can also be four digits and will then be intrepeted
     * as year including a century.
     * 
     * @param txt
     *            String that should be intrepeted as Date
     * @return Date, the result of transforming txt into a new Date
     * @throws NumberFormatException,
     *             thrown if at an expected number was not found at the Expected
     *             position.
     */
    
    public final static Date toDate(String txt) throws NumberFormatException {
        return toDate(txt,m_locale);
    }
    
    public final static Date toDate(String txt, Locale locale) throws NumberFormatException {
        final int HH = 0; // hours, minutes, seconds set to 0;
        final int MM = 0;
        final int SS = 0;
        int dd = 0;
        int mm = -1;
        int yy = 0;
        boolean full_year=false;
        
        if ( (txt.length() == 6 && StringHelper.baseOnly(txt).length() == 6)
                || (txt.length() == 8 && StringHelper.baseOnly(txt).length() == 8)) {
            if (m_locale.equals(Locale.US)) {
                mm = Integer.parseInt(txt.substring(0, 2)) - 1;
                dd = Integer.parseInt(txt.substring(2, 4));
            } else {
                dd = Integer.parseInt(txt.substring(0, 2));
                mm = Integer.parseInt(txt.substring(2, 4)) - 1;
            }
            full_year = (txt.length()-4==4);
            yy = Integer.parseInt(txt.substring(4));
        } else {
            int pos = 0;
            for (int i = 0; i < txt.length(); i++) {
                if (!(Character.isLetterOrDigit(txt.charAt(i)) || ("fr"
                        .equals(locale.getLanguage()) && txt.charAt(i) == '.'))) {
                    if (pos == 0) {
                        dd = Integer.parseInt(txt.substring(pos, i));
                        pos = i + 1;
                    } else {
                        mm = getMonth(txt.substring(pos, i),locale);
                        pos = i + 1;
                    }
                }
            }
            full_year = (txt.length()-pos==4);
            yy = Integer.parseInt(txt.substring(pos));
        }

        if (dd > 0 & mm >= 0 & yy > 0) {
            try {
                if (yy < 100 && !full_year) {
                    yy = yy + ((THIS_YEAR / 100) * 100);
                }
                m_cal.setLenient(false);
                m_cal.set(yy, mm, dd, HH, MM, SS);
                m_cal.set(Calendar.MILLISECOND, 0);
                return m_cal.getTime();
            } catch (IllegalArgumentException iaex) {
                // Lenient causes illegal argument, leave to
                // be handled by NumberFormatException
            }
        }
        throw new NumberFormatException("Not a valid date");
    }

    public static final String isoFromDate(Date d, boolean includeTime) {
        ((SimpleDateFormat) m_sdf.get()).applyPattern(includeTime ? ISO_COMPLETE : ISO_DATE);
        String t = ((SimpleDateFormat) m_sdf.get()).format(d);
        return t;
    }

    public static final String isoFromDate(Date d) {
        return isoFromDate(d, false);
    }

    public static final Date isoToDate(String s) {
        if (s == null)
            return null;
        try {
            int idx = s.indexOf('T');
            boolean includeTime = idx > 0 && s.charAt(idx + 3) == ':';
            ((SimpleDateFormat) m_sdf.get()).applyPattern(includeTime ? ISO_COMPLETE : ISO_DATE);
            return ((SimpleDateFormat) m_sdf.get()).parse(s);
        } catch (ParseException pex) {
            throw new NumberFormatException("Not a valid date");
        }
    }
    
    /**
     * calculate the difference between two {@link Date}s in intervals defined by
     * field.
     * <p>The compare of the calendars follows normal conventions, so if c1 > c2 you will
     * get a negative number. If rounding is on the level below is measured and normal rounding rules apply.</p> 
     * <p>The field value should be one of {@link Calendar#YEAR},{@link Calendar#MONTH},{@link Calendar#DATE}</p>
     * @param d1 Date one (required)
     * @param d2 Date two (required)
     * @param field int, should be in ({@link Calendar#YEAR}|{@link Calendar#MONTH}|{@link Calendar#DATE})
     * @param round boolean, true if the should difference be rounded according to the normal rounding rules.False will floor the value
     * @return int, the difference between the {@link Date}s in the defined measure 
     * @since 1.04.1
     */
    public static final int betweenDate(Date d1, Date d2, int field, boolean round) {
    	if (d1==null || d2==null) throw new IllegalArgumentException("Both dates need to be provided to calculate difference");
    	Calendar c1 = Calendar.getInstance();
    	Calendar c2 = Calendar.getInstance();
    	c1.setTime(d1);
    	c2.setTime(d2);
    	return betweenDate(c1, c2, field, round); 
    }
    
    /**
     * calculate the difference between two {@link Calendar}s in intervals defined by
     * field.
     * <p>The compare of the calendars follows normal conventions, so if c1 > c2 you will
     * get a negative number. If rounding is on the level below is measured and normal rounding rules apply.</p> 
     * <p>The field value should be one of {@link Calendar#YEAR},{@link Calendar#MONTH},{@link Calendar#DATE}</p>
     * @param c1 Calendar one (required)
     * @param c2 Calendar two (required)
     * @param field int, should be in ({@link Calendar#YEAR}|{@link Calendar#MONTH}|{@link Calendar#DATE})
     * @param round boolean, true if the should difference be rounded according to the normal rounding rules.False will floor the value
     * @return int, the difference between the {@link Calendar}s in the defined measure 
     * @since 1.04.1
     */
    public static final int betweenDate(Calendar c1, Calendar c2, int field, boolean round) {
    	if (c1==null || c2==null) throw new IllegalArgumentException("Both calendars need to be provided to calculate difference");
    	double result =0d;
    	switch (field) {
    		case Calendar.YEAR:
    			result = c2.get(Calendar.YEAR) - c1.get(Calendar.YEAR);
    			if (round) {
    				result += (double) (c2.get(Calendar.DAY_OF_YEAR)*c1.getActualMaximum(Calendar.DAY_OF_YEAR)- c1.get(Calendar.DAY_OF_YEAR)*c2.getActualMaximum(Calendar.DAY_OF_YEAR)) / (double) (c2.getActualMaximum(Calendar.DAY_OF_YEAR)* c1.getActualMaximum(Calendar.DAY_OF_YEAR)); 
    			}
    			break;
    		case Calendar.MONTH:
    			result = c2.get(Calendar.MONTH) -c1.get(Calendar.MONTH);
    			result += getYearDiff(c1, c2, Calendar.MONTH);
    			if (round) {
    				result += (double) (c2.get(Calendar.DAY_OF_MONTH)*c1.getActualMaximum(Calendar.DAY_OF_MONTH)- c1.get(Calendar.DAY_OF_MONTH)*c2.getActualMaximum(Calendar.DAY_OF_MONTH)) / (double) (c2.getActualMaximum(Calendar.DAY_OF_MONTH)* c1.getActualMaximum(Calendar.DAY_OF_MONTH)); 
    			}
    			break;
    		case Calendar.DATE:
    		case Calendar.DAY_OF_YEAR:
    			result = c2.get(Calendar.DAY_OF_YEAR) - c1.get(Calendar.DAY_OF_YEAR);
    			result += getYearDiff(c1, c2, Calendar.DAY_OF_YEAR);
    			if (round) {
    				result += (double) (c2.get(Calendar.HOUR_OF_DAY)*c1.getActualMaximum(Calendar.HOUR_OF_DAY) - c1.get(Calendar.HOUR_OF_DAY)*c2.getActualMaximum(Calendar.HOUR_OF_DAY)) / (double) (c2.getActualMaximum(Calendar.HOUR_OF_DAY)* c1.getActualMaximum(Calendar.HOUR_OF_DAY));
    			}
    			break;
    		default:
    			throw new IllegalArgumentException("Cannot determine difference for the supplied field");
    	}
    	
    	return (int) Math.round(result);
    }
    
    private final static int getYearDiff(Calendar c1, Calendar c2, int field) {
    	int result=0;
    	int start = c1.get(Calendar.YEAR);
		int end = c2.get(Calendar.YEAR);
		Calendar clone = (Calendar) c1.clone();
		if (start>end) {
			start = end;
			end = c1.get(Calendar.YEAR);
			clone = (Calendar) c2.clone();
		}
		for (int i=start;i<end;i++) {
			result+=clone.getActualMaximum(field);
			clone.set(Calendar.YEAR, i);
		}
    	return result;
    }

    /**
     * gets the Month as int (0 to 11) from a String that either represents a
     * numerical month, or a String that represents a long or short month in the
     * language of the default Locale
     * 
     * @param str
     *            Stirng, the possible Month
     * @return int, the month (0 to 11) if one could be established, otherwise
     *         -1
     */
    private static int getMonth(String str, Locale locale) {
        int mm = 0;
        try {
            mm = Integer.parseInt(str) - 1;
            return mm;
        } catch (NumberFormatException ex) {
            DateFormatSymbols symb = new DateFormatSymbols(locale);
            if ((str.length() > 3 && !"fr".equals(locale.getLanguage()))
                    || ("fr".equals(locale.getLanguage()) && !str
                            .endsWith("."))) {
                String[] months = symb.getMonths();
                for (int i = 0; i < months.length; i++) {
                    if (str.equalsIgnoreCase(months[i])) {
                        return i;
                    }
                }
            } else {
                String[] months = symb.getShortMonths();
                for (int i = 0; i < months.length; i++) {
                    if (str.equalsIgnoreCase(months[i])) {
                        return i;
                    }
                }
            }
        }
        return -1;
    }
}
